﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Security.Permissions;
using HeuristicLab.Hive.Contracts;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.Hive.Contracts.ResponseObjects;
using HeuristicLab.Hive.Server.DataAccess;
using System.ServiceModel;

namespace HeuristicLab.Hive.Server.Core {
  [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall, ConcurrencyMode = ConcurrencyMode.Multiple)]
  public class ServerConsoleFacade : IServerConsoleFacade {
    private ISlaveManager slaveManager = ServiceLocator.GetSlaveManager();

    private IJobManager jobManager = ServiceLocator.GetJobManager();

    private IContextFactory contextFactory = ServiceLocator.GetContextFactory();

    public Guid sessionID = Guid.Empty;

    public ServerConsoleFacade() {
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response Login() {
      Response resp = new Response();
      return resp;
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseList<SlaveDto> GetAllSlaves() {
      using (contextFactory.GetContext(false)) {
        return slaveManager.GetAllSlaves();
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseList<SlaveGroupDto> GetAllSlaveGroups() {
      using (contextFactory.GetContext(false)) {
        return slaveManager.GetAllSlaveGroups();
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseList<UpTimeStatisticsDto> GetAllUpTimeStatistics() {
      using (contextFactory.GetContext(false)) {
        return slaveManager.GetAllUpTimeStatistics();
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<SlaveGroupDto> AddSlaveGroup(SlaveGroupDto slaveGroup) {
      using (contextFactory.GetContext()) {
        return slaveManager.AddSlaveGroup(slaveGroup);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response AddResourceToGroup(Guid slaveGroupId, ResourceDto resource) {
      using (contextFactory.GetContext()) {
        return slaveManager.AddResourceToGroup(slaveGroupId, resource);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response DeleteResourceFromGroup(Guid slaveGroupId, Guid resourceId) {
      using (contextFactory.GetContext()) {
        return slaveManager.DeleteResourceFromGroup(slaveGroupId, resourceId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseList<JobDto> GetAllJobs() {
      using (contextFactory.GetContext(false)) {
        return jobManager.GetAllJobs();
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseList<JobDto> GetAllJobsWithFilter(JobState jobState, int offset, int count) {
      using (contextFactory.GetContext(false)) {
        return jobManager.GetAllJobsWithFilter(jobState, offset, count);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<JobDto> GetJobById(Guid jobId) {
      using (contextFactory.GetContext(false)) {
        return jobManager.GetJobById(jobId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<JobDto> GetJobByIdWithDetails(Guid jobId) {
      using (contextFactory.GetContext(false)) {
        return jobManager.GetJobByIdWithDetails(jobId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<JobDto> AddNewJob(SerializedJob job) {
      using (contextFactory.GetContext()) {
        return jobManager.AddNewJob(job);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<SerializedJob> GetLastSerializedResult(Guid jobId) {
      using (contextFactory.GetContext(false)) {
        return jobManager.GetLastSerializedResult(jobId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<SerializedJob> GetSnapshotResult(Guid jobId) {
      using (contextFactory.GetContext(false)) {
        return jobManager.GetSnapshotResult(jobId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<JobResultList> GetJobResults(IEnumerable<Guid> jobIds) {
      using (contextFactory.GetContext(false)) {
        return jobManager.GetJobResults(jobIds);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response RequestSnapshot(Guid jobId) {
      using (contextFactory.GetContext()) {
        return jobManager.RequestSnapshot(jobId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response AbortJob(Guid jobId) {
      using (contextFactory.GetContext()) {
        return jobManager.AbortJob(jobId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<SlaveGroupDtoList> GetAllGroupsOfResource(Guid resourceId) {
      using (contextFactory.GetContext(false)) {
        return slaveManager.GetAllGroupsOfResource(resourceId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response DeleteSlaveGroup(Guid slaveGroupId) {
      using (contextFactory.GetContext()) {
        return slaveManager.DeleteSlaveGroup(slaveGroupId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseList<ProjectDto> GetAllProjects() {
      using (contextFactory.GetContext(false)) {
        return jobManager.GetAllProjects();
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response CreateProject(ProjectDto project) {
      using (contextFactory.GetContext()) {
        return jobManager.CreateProject(project);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response ChangeProject(ProjectDto project) {
      using (contextFactory.GetContext()) {
        return jobManager.ChangeProject(project);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response DeleteProject(Guid projectId) {
      using (contextFactory.GetContext()) {
        return jobManager.DeleteProject(projectId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseList<JobDto> GetJobsByProject(Guid projectId) {
      using (contextFactory.GetContext(false)) {
        return jobManager.GetJobsByProject(projectId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseList<AppointmentDto> GetUptimeCalendarForResource(Guid guid) {
      using (contextFactory.GetContext(false)) {
        return slaveManager.GetUptimeCalendarForResource(guid);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response SetUptimeCalendarForResource(Guid guid, IEnumerable<AppointmentDto> appointments, bool isForced) {
      using (contextFactory.GetContext()) {
        return slaveManager.SetUptimeCalendarForResource(guid, appointments, isForced);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<JobDto> AddJobWithGroupStrings(SerializedJob job, IEnumerable<string> resources) {
      using (contextFactory.GetContext()) {
        return jobManager.AddJobWithGroupStrings(job, resources);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<JobResultList> GetChildJobResults(Guid? parentJobId, bool recursive, bool includeParent) {
      using (contextFactory.GetContext(false)) {
        return jobManager.GetChildJobResults(parentJobId, recursive, includeParent);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<JobDto> AddChildJob(Guid parentJobId, SerializedJob serializedJob) {
      using (contextFactory.GetContext()) {
        return jobManager.AddChildJob(parentJobId, serializedJob);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<JobDto> PauseJob(SerializedJob serializedJob) {
      using (contextFactory.GetContext()) {
        return jobManager.PauseJob(serializedJob);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<HiveExperimentDtoList> GetHiveExperiments() {
      using (contextFactory.GetContext(false)) {
        return jobManager.GetHiveExperiments();
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public ResponseObject<HiveExperimentDto> UpdateHiveExperiment(HiveExperimentDto hiveExperimentDto) {
      using (contextFactory.GetContext()) {
        return jobManager.UpdateHiveExperiment(hiveExperimentDto);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response DeleteHiveExperiment(Guid hiveExperimentId) {
      using (contextFactory.GetContext()) {
        return jobManager.DeleteHiveExperiment(hiveExperimentId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response DeleteChildJobs(Guid jobId) {
      using (contextFactory.GetContext()) {
        return jobManager.DeleteChildJobs(jobId);
      }
    }

    [PrincipalPermission(SecurityAction.Demand, Role = HiveRoles.Administrator)]
    public Response DeleteJob(Guid jobId) {
      using (contextFactory.GetContext()) {
        return jobManager.DeleteJob(jobId);
      }
    }
  }
}
